package com.mca.map.mapper;

import java.util.List;

import com.mca.map.vo.UseRequestSearchVO;
import com.mca.map.vo.UseRequestVO;

import egovframework.rte.psl.dataaccess.mapper.Mapper;

@Mapper("useReqeustMapper")
public interface UseReqeustMapper {
	
	
	public int selectUseRequestCnt(UseRequestSearchVO useRequestSearchVO);	

	public List<?> selectUseRequestList(UseRequestSearchVO useRequestSearchVO);

	public void insertReq(UseRequestVO useRequestVO);

	public UseRequestVO selectUseRequest(int idx);

	public void updateReq(UseRequestVO useRequestVO);

	public void deleteReq(int idx);

	public void updateReqConfirm(UseRequestVO useRequestVO);

	public int selectUseRequeststatusCnt();
}
